<?php

namespace Drupal\entity_hierarchy\Plugin\views\argument;

use Drupal\Core\Form\FormStateInterface;
use Drupal\entity_hierarchy\Storage\Record;
use Drupal\views\Plugin\views\query\Sql;

/**
 * Argument to limit to parent of an entity.
 *
 * @ingroup views_argument_handlers
 *
 * @ViewsArgument("entity_hierarchy_argument_is_sibling_of_entity")
 */
class HierarchyIsSiblingOfEntity extends EntityHierarchyArgumentPluginBase {

  /**
   * Set up the query for this argument.
   *
   * The argument sent may be found at $this->argument.
   */
  public function query($group_by = FALSE) {
    $this->ensureMyTable();
    // Load the actual entity.
    $filtered = FALSE;
    if ($entity = $this->loadEntity()) {
      $queryBuilder = $this->queryBuilderFactory->get($this->configuration['entity_hierarchy_field'], $entity->getEntityTypeId());
      $parent = $queryBuilder->findParent($entity);
      $records = $queryBuilder->findDescendants($parent, 1);
      if (!$this->options['show_self']) {
        $records = $records->filter(fn (Record $node) => $node->getId() !== (int) $entity->id());
      }
      $children = $records->map(fn (Record $node) => $node->getId());
      if (!$children) {
        $this->query->addWhereExpression(0, '1 <> 1');
        return;
      }
      assert($this->query instanceof Sql);
      $this->query->addWhere(0, "$this->tableAlias.$this->realField", $children, 'IN');
      $filtered = TRUE;
    }
    // The parent entity doesn't exist, or isn't in the tree and hence has no
    // children.
    if (!$filtered) {
      // Add a killswitch.
      $this->query->addWhereExpression(0, '1 <> 1');
    }
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state): void {
    $form['show_self'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show self'),
      '#default_value' => $this->options['show_self'],
      '#description' => $this->t('Filter out the current child from the list of siblings.'),
    ];
    parent::buildOptionsForm($form, $form_state);
    unset($form['depth']);
  }

  /**
   * {@inheritdoc}
   */
  protected function defineOptions(): array {
    $options = parent::defineOptions();
    unset($options['depth']);
    $options['show_self'] = ['default' => FALSE];

    return $options;
  }

}
