<?php

declare(strict_types=1);

namespace Drupal\entity_hierarchy\Plugin\views\join;

use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\entity_hierarchy\Query\HierarchyInfo as HierarchyInfoQuery;
use Drupal\views\Plugin\views\join\JoinPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines a join plugin for hierarchy info for a given field.
 *
 * @ingroup views_join_handlers
 * @ViewsJoin("entity_hierarchy")
 */
final class HierarchyInfo extends JoinPluginBase implements ContainerFactoryPluginInterface {

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): static {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('entity_field.manager'),
      $container->get('database')
    );
  }

  /**
   * Constructs a new HierarchyInfo.
   *
   * @param array $configuration
   *   Config.
   * @param string $plugin_id
   *   ID.
   * @param array $plugin_definition
   *   Definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   Entity type manager.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entityFieldManager
   *   Entity field manager.
   * @param \Drupal\Core\Database\Connection $database
   *   Database.
   */
  public function __construct(
    array $configuration,
    string $plugin_id,
    array $plugin_definition,
    protected EntityTypeManagerInterface $entityTypeManager,
    protected EntityFieldManagerInterface $entityFieldManager,
    protected Connection $database
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $query = new HierarchyInfoQuery($this->database, $this->entityFieldManager->getFieldStorageDefinitions($configuration['entity_type_id'])[$configuration['field_name']], $this->entityTypeManager);

    $this->configuration['type'] = 'LEFT';
    $this->configuration['table formula'] = $query;
    $this->configuration['field'] = 'id';
    $this->configuration['adjusted'] = TRUE;
  }

}
