<?php

declare(strict_types=1);

namespace Drupal\entity_hierarchy\Query;

use Drupal\Core\Database\Connection;
use Drupal\Core\Database\Query\PlaceholderInterface;
use Drupal\Core\Database\Query\Query;
use Drupal\Core\Database\Query\SelectInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\entity_hierarchy\Storage\QueryBuilderTrait;

/**
 * Defines a base class for CTE queries.
 */
abstract class TreeSelectBase extends Query implements SelectInterface {

  use QueryBuilderTrait;

  /**
   * Constructs a Select object.
   *
   * @param \Drupal\Core\Database\Connection $connection
   *   Database connection object.
   * @param \Drupal\Core\Field\FieldStorageDefinitionInterface $fieldStorageDefinition
   *   Field storage definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   Entity type manager.
   */
  public function __construct(Connection $connection, protected FieldStorageDefinitionInterface $fieldStorageDefinition, protected EntityTypeManagerInterface $entityTypeManager) {
    parent::__construct($connection, []);
    $this->setupTablesAndColumns();
  }

  /**
   * {@inheritdoc}
   */
  public function addTag($tag) {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function hasTag($tag) {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function hasAllTags() {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function hasAnyTag() {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function addMetaData($key, $object) {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function getMetaData($key) {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function condition($field, $value = NULL, $operator = '=') {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function where($snippet, $args = []) {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function isNull($field) {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function isNotNull($field) {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function exists(SelectInterface $select) {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function notExists(SelectInterface $select) {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function alwaysFalse() {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function &conditions() {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function arguments(): array {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function compile(Connection $connection, PlaceholderInterface $queryPlaceholder) {
  }

  /**
   * {@inheritdoc}
   */
  public function compiled() {
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function conditionGroupFactory($conjunction = 'AND') {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function andConditionGroup() {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function orConditionGroup() {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function extend($extender_name) {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function execute() {
    return $this->connection->query((string) $this, [], $this->queryOptions);
  }

  /**
   * {@inheritdoc}
   */
  public function &getFields() {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function &getExpressions() {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function &getOrderBy() {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function &getGroupBy() {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function &getTables() {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function &getUnion() {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function escapeLike($string): string {
    return $this->connection->escapeLike($string);
  }

  /**
   * {@inheritdoc}
   */
  public function escapeField($string): string {
    return $this->connection->escapeLike($string);
  }

  /**
   * {@inheritdoc}
   */
  public function getArguments(PlaceholderInterface $queryPlaceholder = NULL) {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function distinct($distinct = TRUE) {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function addField($table_alias, $field, $alias = NULL) {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function fields($table_alias, array $fields = []) {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function addExpression($expression, $alias = NULL, $arguments = []) {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function join($table, $alias = NULL, $condition = NULL, $arguments = []) {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function innerJoin($table, $alias = NULL, $condition = NULL, $arguments = []) {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function leftJoin($table, $alias = NULL, $condition = NULL, $arguments = []) {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function addJoin($type, $table, $alias = NULL, $condition = NULL, $arguments = []) {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function orderBy($field, $direction = 'ASC') {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function orderRandom() {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function range($start = NULL, $length = NULL) {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function union(SelectInterface $query, $type = '') {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function groupBy($field) {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function countQuery() {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function isPrepared(): bool {
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function preExecute(SelectInterface $query = NULL): bool {
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function havingCondition($field, $value = NULL, $operator = NULL) {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function &havingConditions() {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function havingArguments() {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function having($snippet, $args = []) {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function havingCompile(Connection $connection) {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function havingIsNull($field) {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function havingIsNotNull($field) {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function havingExists(SelectInterface $select) {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function havingNotExists(SelectInterface $select) {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * {@inheritdoc}
   */
  public function forUpdate($set = TRUE) {
    throw new \BadMethodCallException('Unsupported');
  }

  /**
   * Generate the query for this select.
   *
   * @return string
   *   Query string.
   */
  abstract public function __toString(): string;

}
