<?php

declare(strict_types=1);

namespace Drupal\entity_hierarchy\Update;

use Drupal\views\ViewEntityInterface;
use Drupal\views\ViewsConfigUpdater as BaseViewsConfigUpdater;

/**
 * Defines a class for updating views config.
 *
 * @phpstan-ignore-next-line
 */
final class ViewsConfigUpdater extends BaseViewsConfigUpdater {

  /**
   * Updates entity hierarchy handlers.
   *
   * @param \Drupal\views\ViewEntityInterface $view
   *   View to update.
   *
   * @return bool
   *   TRUE if changes were made.
   */
  public function updateEntityHierarchyHandlers(ViewEntityInterface $view): bool {
    $base_table = $view->get('base_table');
    $data = $this->viewsData->get($base_table);
    $entity_type_id = $data['table']['entity type'] ?? NULL;
    if (!$entity_type_id) {
      return FALSE;
    }
    $entity_type = $this->entityTypeManager->getDefinition($entity_type_id);
    $target_base_table = $this->entityTypeManager->getHandler($entity_type_id, 'views_data')->getViewsTableForEntityType($entity_type);
    $field_names = array_keys($this->entityFieldManager->getFieldMapByFieldType('entity_reference_hierarchy')[$entity_type_id] ?? []);
    if (empty($field_names)) {
      return FALSE;
    }
    return $this->processDisplayHandlers($view, FALSE, function (&$handler, $handler_type, $key, $display_id) use ($entity_type_id, $target_base_table) {
      if ($handler_type !== 'relationship' || $handler['plugin_id'] !== 'entity_hierarchy_root' || $handler['table'] !== $target_base_table) {
        return FALSE;
      }
      [, , $field_name] = explode('__', $handler['field'], 3);
      $table_name = sprintf('nested_set_%s_%s', $field_name, $entity_type_id);
      $handler['plugin_id'] = 'standard';
      $handler['field'] = 'root_id';
      $handler['table'] = $table_name;
      $handler['id'] = 'root_id';
      return TRUE;
    });
  }

}
