<?php

declare(strict_types=1);

namespace Drupal\Tests\entity_hierarchy\Functional;

use Behat\Mink\Element\NodeElement;
use Drupal\FunctionalTests\Update\UpdatePathTestBase;

/**
 * Tests update path for views integration between 3.x and 5.x branches.
 *
 * @group entity_hierarchy
 */
final class ViewsUpdatePathTest extends UpdatePathTestBase {

  /**
   * {@inheritdoc}
   */
  protected function setDatabaseDumpFiles(): void {
    $this->databaseDumpFiles = [
      dirname(__DIR__, 2) . '/fixtures/3284026-dump.sql.gz',
    ];
  }

  /**
   * Tests update path.
   */
  public function testUpdatePath(): void {
    $hierarchy_config = \Drupal::config('views.view.hierarchy');
    $this->assertEquals('nested_set_field_parent_node', $hierarchy_config->get('display.default.display_options.sorts.left_pos.table'));
    $this->assertEquals('left_pos', $hierarchy_config->get('display.default.display_options.sorts.left_pos.field'));
    $this->assertEquals('standard', $hierarchy_config->get('display.default.display_options.sorts.left_pos.plugin_id'));
    $this->assertEquals('nested_set_field_parent_node', $hierarchy_config->get('display.default.display_options.sorts.depth.table'));
    $this->assertEquals('depth', $hierarchy_config->get('display.default.display_options.sorts.depth.field'));
    $this->assertEquals('standard', $hierarchy_config->get('display.default.display_options.sorts.depth.plugin_id'));
    $this->assertEquals('entity_hierarchy_root', $hierarchy_config->get('display.default.display_options.relationships.top__node__field_parent.plugin_id'));
    $this->assertEquals('node_field_data', $hierarchy_config->get('display.default.display_options.relationships.top__node__field_parent.table'));
    $this->assertEquals('top__node__field_parent', $hierarchy_config->get('display.default.display_options.relationships.top__node__field_parent.field'));
    $this->assertEquals('top__node__field_parent', $hierarchy_config->get('display.default.display_options.relationships.top__node__field_parent.id'));

    $siblings_config = \Drupal::config('views.view.siblings');
    $this->assertEquals('is_sibling', $siblings_config->get('display.default.display_options.arguments.is_sibling.field'));
    $this->assertEquals('nested_set_field_collection_node', $siblings_config->get('display.default.display_options.arguments.is_sibling.table'));
    $this->assertEquals('entity_hierarchy_argument_is_sibling_of_entity_revision', $siblings_config->get('display.default.display_options.arguments.is_sibling.plugin_id'));

    $parents_config = \Drupal::config('views.view.parent');
    $this->assertEquals('is_parent', $parents_config->get('display.default.display_options.arguments.is_parent.field'));
    $this->assertEquals('nested_set_field_collection_node', $parents_config->get('display.default.display_options.arguments.is_parent.table'));
    $this->assertEquals('entity_hierarchy_argument_is_parent_of_entity_revision', $parents_config->get('display.default.display_options.arguments.is_parent.plugin_id'));

    $children_config = \Drupal::config('views.view.children');
    $this->assertEquals('is_child', $children_config->get('display.default.display_options.arguments.is_child.field'));
    $this->assertEquals('nested_set_field_collection_node', $children_config->get('display.default.display_options.arguments.is_child.table'));
    $this->assertEquals('entity_hierarchy_argument_is_child_of_entity_revision', $children_config->get('display.default.display_options.arguments.is_child.plugin_id'));

    $this->runUpdates();

    $this->doTestViewsFunctionality();

    $hierarchy_config = \Drupal::config('views.view.hierarchy');
    // Should be unchanged.
    $this->assertEquals('nested_set_field_parent_node', $hierarchy_config->get('display.default.display_options.sorts.left_pos.table'));
    $this->assertEquals('left_pos', $hierarchy_config->get('display.default.display_options.sorts.left_pos.field'));
    $this->assertEquals('standard', $hierarchy_config->get('display.default.display_options.sorts.left_pos.plugin_id'));
    $this->assertEquals('nested_set_field_parent_node', $hierarchy_config->get('display.default.display_options.sorts.depth.table'));
    $this->assertEquals('depth', $hierarchy_config->get('display.default.display_options.sorts.depth.field'));
    $this->assertEquals('standard', $hierarchy_config->get('display.default.display_options.sorts.depth.plugin_id'));
    // Should be updated.
    $this->assertEquals('standard', $hierarchy_config->get('display.default.display_options.relationships.top__node__field_parent.plugin_id'));
    $this->assertEquals('nested_set_field_parent_node', $hierarchy_config->get('display.default.display_options.relationships.top__node__field_parent.table'));
    $this->assertEquals('root_id', $hierarchy_config->get('display.default.display_options.relationships.top__node__field_parent.field'));
    $this->assertEquals('root_id', $hierarchy_config->get('display.default.display_options.relationships.top__node__field_parent.id'));

    // The rest should be unchanged.
    $siblings_config = \Drupal::config('views.view.siblings');
    $this->assertEquals('is_sibling', $siblings_config->get('display.default.display_options.arguments.is_sibling.field'));
    $this->assertEquals('nested_set_field_collection_node', $siblings_config->get('display.default.display_options.arguments.is_sibling.table'));
    $this->assertEquals('entity_hierarchy_argument_is_sibling_of_entity_revision', $siblings_config->get('display.default.display_options.arguments.is_sibling.plugin_id'));

    $parents_config = \Drupal::config('views.view.parent');
    $this->assertEquals('is_parent', $parents_config->get('display.default.display_options.arguments.is_parent.field'));
    $this->assertEquals('nested_set_field_collection_node', $parents_config->get('display.default.display_options.arguments.is_parent.table'));
    $this->assertEquals('entity_hierarchy_argument_is_parent_of_entity_revision', $parents_config->get('display.default.display_options.arguments.is_parent.plugin_id'));

    $children_config = \Drupal::config('views.view.children');
    $this->assertEquals('is_child', $children_config->get('display.default.display_options.arguments.is_child.field'));
    $this->assertEquals('nested_set_field_collection_node', $children_config->get('display.default.display_options.arguments.is_child.table'));
    $this->assertEquals('entity_hierarchy_argument_is_child_of_entity_revision', $children_config->get('display.default.display_options.arguments.is_child.plugin_id'));
  }

  /**
   * Assert functionality of views plugins.
   */
  protected function doTestViewsFunctionality(): void {
    $assert = $this->assertSession();

    // Siblings and parent arguments.
    $this->drupalGet('/node/7');
    $assert->elementExists('css', 'h1:contains("Article 1.2")');
    $assert->elementExists('css', 'h2:contains("Parent")');
    $assert->elementExists('css', 'h2:contains("Siblings")');
    $assert->elementExists('named_exact', ['link', 'Article 1.1']);
    $assert->elementNotExists('named_exact', ['link', 'Article 2.1']);
    $assert->elementExists('named_exact', ['link', 'Collection 1']);
    $assert->elementNotExists('named_exact', ['link', 'Collection 2']);

    // Children argument.
    $this->drupalGet('/node/4');
    $assert->elementExists('css', 'h1:contains("Collection 1")');
    $assert->elementExists('css', 'h2:contains("Children")');
    $assert->elementExists('named_exact', ['link', 'Article 1.1']);
    $assert->elementExists('named_exact', ['link', 'Article 1.2']);
    $assert->elementNotExists('named_exact', ['link', 'Article 2.1']);

    // Hierarchy order, depth order, depth filter, root relationship, depth
    // field.
    $this->drupalGet('/node/9');
    $assert->elementExists('css', 'h1:contains("Root 2")');
    $assert->elementExists('css', 'h2:contains("Hierarchy")');
    $table = $assert->elementExists('css', 'table');
    $data = array_map(
      fn(NodeElement $row) => array_map(
        fn(NodeElement $cell) => $cell->getText(),
        $row->findAll('css', 'td,th')),
      $table->findAll('css', 'tr')
    );
    $this->assertEquals([
      ['Title', 'Depth', 'Root'],
      ['Grandparent', 1, 'Root'],
      ['Parent', 2, 'Root'],
      ['Parent 2.1', 1, 'Root 2'],
      ['Parent 2.1.1', 2, 'Root 2'],
      ['Child 2.1.1.1', 3, 'Root 2'],
      ['Parent 2.1.2', 2, 'Root 2'],
      ['Child 2.1.2.1', 3, 'Root 2'],
      ['Parent 2.2', 1, 'Root 2'],
      ['Parent 2.2.1', 2, 'Root 2'],
    ], $data);
  }

}
